import glob
import logging
import os
import re
import time

import click
import emoji
from watchdog.events import PatternMatchingEventHandler
from watchdog.observers import Observer


def formatting_engine(lines_list):
    computed_list = [re.sub(r"=.*", "=", line) for line in lines_list]
    return computed_list


def sync_dot_env(
    source,
    destination_env,
    destination_envs=None,
    just_variables=False,
):
    computed_list = []
    lines_list = []

    with open(f"{source}", "r") as f:
        lines_list = f.readlines()
        computed_list = formatting_engine(lines_list)

    if destination_envs is not None:
        for filename in glob.glob(
            os.path.join(destination_envs, ".env*"), recursive=False
        ):
            with open(f"{filename}", "w") as f:
                if just_variables:
                    f.writelines(computed_list)
                else:
                    f.writelines(lines_list)

    if destination_env is not None:
        with open(f"{destination_env}", "w") as f:
            if just_variables:
                f.writelines(computed_list)
            else:
                f.writelines(lines_list)


@click.group(invoke_without_command=True)
@click.option(
    "-s",
    "--source",
    default=".env",
    show_default=True,
    type=click.Path(exists=True, file_okay=True, dir_okay=False, readable=True),
    help="source .env file to use in populating other .env files",
)
@click.option(
    "-d",
    "--destination-env",
    required=False,
    type=click.Path(
        exists=True, file_okay=True, dir_okay=False, readable=True, writable=True
    ),
    help=".env file for destination stage",
)
@click.option(
    "-ds",
    "--destination-envs",
    type=click.Path(exists=True, file_okay=False, dir_okay=True, readable=True),
    help="directory path to .env files for destination stage",
)
@click.option(
    "--just-variables",
    is_flag=True,
    help="indicates to the engine to synchronize just the variables",
)
@click.pass_context
def main(
    context,
    source,
    destination_env,
    destination_envs,
    just_variables,
    *args,
    **kwargs,
):
    """This script synchronizes .env files."""

    if context.invoked_subcommand is None:
        click.echo(
            emoji.emojize(
                ":loudspeaker: "
                + click.style(
                    "Synchronizing your .env file(s)",
                    bold=True,
                )
                + " :loudspeaker:"
            )
        )
        sync_dot_env(
            source=source,
            destination_env=destination_env,
            destination_envs=destination_envs,
            just_variables=just_variables,
        )
        click.echo(
            emoji.emojize(":fire: Synchronizing Complete :fire:", use_aliases=True)
        )
    else:
        click.echo(
            emoji.emojize(
                ":eye: "
                + click.style(
                    "Watching for source .env file changes",
                    bold=True,
                )
                + " :eye:"
            )
        )
        click.echo("Quit the watch with CONTROL-C.")
        context.ensure_object(dict)
        context.obj["source"] = source
        context.obj["destination_env"] = destination_env
        context.obj["destination_envs"] = destination_envs
        context.obj["just_variables"] = just_variables


def watch_sync_dot_env(**kwargs):
    sync_dot_env(**kwargs)
    click.echo(emoji.emojize(":fire: .env file(s) updated :fire:", use_aliases=True))


@main.command()
@click.option(
    "--show-logs",
    is_flag=True,
    help="indicates logging of file changes events",
)
@click.pass_context
def watch(context, show_logs):
    """This command watches the source .env file for changes & synchronize on change."""
    if show_logs:
        logging.basicConfig(
            level=logging.INFO,
            format="%(asctime)s - %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S",
        )
    src_path = os.getcwd()
    event_handler = PatternMatchingEventHandler(
        patterns=[f"{context.obj['source']}"],
        ignore_directories=True,
        case_sensitive=False,
        ignore_patterns=None,
    )
    event_handler.on_modified = lambda event: watch_sync_dot_env(**context.obj)
    observer = Observer()
    observer.schedule(event_handler, src_path)
    observer.start()
    try:
        while True:
            time.sleep(1)
    finally:
        observer.stop()
        observer.join()



import glob
import os

import emoji
import pytest
from click.testing import CliRunner

from py_sync_dotenv.cli import formatting_engine, main

test_env = """
PRODUCTION=1
DEBUG=0
SECRET_KEY=iu0^cv$mghjfa_vp8vk)e(c_5^cfo7staccjs4+!f#=1a_22-h2
DJANGO_ALLOWED_HOSTS=localhost 127.0.0.1 [::1]
"""


@pytest.mark.parametrize(
    "source, dev_env, dev_envs, just_variables",
    [
        ("", "", "", ""),
        ("-s .env", "", "", ""),
        ("-s .env", "-d .env.dev", "", ""),
        ("-s .env", "-d .env.dev", "-ds dev_envs/", ""),
        ("-s .env", "-d .env.dev", "-ds dev_envs/", "--just-variables"),
    ],
)
def test_standard_sync(source, dev_env, dev_envs, just_variables):
    runner = CliRunner()
    with runner.isolated_filesystem():
        s_source = source.split(" ")[1] if source else ".env"
        with open(f"{s_source}", "w") as f:
            f.write(test_env)

        if dev_env:
            with open(f"{dev_env.split(' ')[1]}", "a") as f:
                pass

        if dev_envs:
            if not os.path.exists(dev_envs.split(" ")[1]):
                os.makedirs(dev_envs.split(" ")[1])
                with open(f"{dev_envs.split(' ')[1]}/.env.dev", "a") as f:
                    pass
                with open(f"{dev_envs.split(' ')[1]}/.env1.dev", "a") as f:
                    pass

        args = f"{source} {dev_env} {dev_envs} {just_variables}"
        result = runner.invoke(main, args=args)
        assert not result.exception
        assert result.output == (
            f"{emoji.emojize(':loudspeaker: Synchronizing your .env file(s) :loudspeaker:')}"
            + f"\n{emoji.emojize(':fire: Synchronizing Complete :fire:', use_aliases=True)}\n"
        )

        if dev_env:
            with open(f"{dev_env.split(' ')[1]}", "r") as f:
                output = f.read()
                if just_variables:
                    assert output == "\n".join(formatting_engine(test_env.split("\n")))
                else:
                    assert output == test_env

        if dev_envs:
            for filename in glob.glob(
                os.path.join(dev_envs.split(" ")[1], ".env*"), recursive=False
            ):
                with open(f"{filename}", "r") as f:
                    output = f.read()
                    if just_variables:
                        assert output == "\n".join(
                            formatting_engine(test_env.split("\n"))
                        )
                    else:
                        assert output == test_env


@pytest.mark.parametrize(
    "source, dev_env, dev_envs, just_variables",
    [
        ("", "", "", ""),
        ("-s .env", "", "", ""),
        ("-s .env", "-d .env.dev", "", ""),
        ("-s .env", "-d .env.dev", "-ds dev_envs/", ""),
        ("-s .env", "-d .env.dev", "-ds dev_envs/", "--just-variables"),
    ],
)
def test_standard_sync_invalid_path(source, dev_env, dev_envs, just_variables):
    runner = CliRunner()
    with runner.isolated_filesystem():
        args = f"{source} {dev_env} {dev_envs} {just_variables}"
        result = runner.invoke(main, args=args)
        assert result.exception
        assert result.exit_code == 2


def test_standard_sync_invlaid_source_file():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(main, "-s .env")
        assert result.exception
        assert result.exit_code == 2


def test_standard_sync_invlaid_dev_envs_dir():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(main, "-ds dev_envs/")
        assert result.exception
        assert result.exit_code == 2


def test_standard_sync_invlaid_prod_envs_dir():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(main, "-ps prod_envs/")
        assert result.exception
        assert result.exit_code == 2



#!/usr/bin/env python

"""The setup script."""

from setuptools import find_packages, setup

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

requirements = ["Click>=7.0", "colorama>=0.4", "watchdog>=2.0", "emoji>=1.2"]

setup_requirements = [
    "pytest-runner",
]

test_requirements = [
    "pytest>=3",
]

setup(
    author="Sotunde Abiodun Oladimeji",
    author_email="sotundeabiodun00@gmail.com",
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="Python Sync Dotenv is a Python package for synchronizing .env files  across projects.",
    entry_points={
        "console_scripts": [
            "py-sync-dotenv=py_sync_dotenv.cli:main",
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="py_sync_dotenv",
    name="py_sync_dotenv",
    packages=find_packages(include=["py_sync_dotenv", "py_sync_dotenv.*"]),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/IamAbbey/py_sync_dotenv",
    version="0.1.0",
    zip_safe=False,
)
